//
// Implementation file for module (implements any functions/services provided by the module. 
// In this file, the actual definition of variables and implementations of the functions defined in the header file should be included.
//
// Modified to allow fractional box counts, based on a modified version of Sec. 3.2 of
// So, So and Jin, Pattern Recongition Letters 98, 53 (2017).
//

#include "BoxCount.h" //must include header file that for referred to functions 


//
// Output to file.
//
void DataToFile (int r, double num){ //fn for writing the box counting data to an external file.
  FILE *DataOutput; //FILE is a structure which must be pointed to
  DataOutput = fopen("BoxCountData.csv", "a"); //write to and open a new file called BoxCountData. Beware this writes over any file with this name in the directory! "a" appends data on new run to file (with a new line space)
  if (!DataOutput){
    printf("Close open .csv file first!");
    return;
  }

	if (r==-1)
		fprintf(DataOutput, "r, num\n"); //to print header at start of file. It will do this on the first loop of a run of the programme, as this is the initial condition in called function.
	else if (r==0)
		fprintf(DataOutput, "\n"); //add new line at end of file (to separate appended data).
	else 
  		fprintf(DataOutput, "%i, %g\n", r, num); // for writing to a .csv file: 
  
  fclose(DataOutput); //close file pointed to to avoid loss of data or runtime errors

}


//
// Initialises the cell counting array to the given box size. "nb" is "numBoxes".
// This is an optimisation that works the same as the old 'OccElementCount' function, but reduces
// the number of loopers required and so is faster.
//
void initCellCountArray( int nb )
{
  int i, j, k;

  // 'cellCountArray' is a variable (pointer, really) for this module declared in the .h file.
  // Initialise to a 3D array, with numBoxes+1 for each side - note the "+1" is to include any
  // partial boxes that intersect the simulation boundary.
  cellCountArray = (int*) malloc( (nb+1)*(nb+1)*(nb+1) * sizeof(int) );

  // Check this memory allocation was successful. Hard to imagine it would fail, but nice to check.
  if( !cellCountArray )
  {
    printf( "FATAL: Could not allocate memory for the 'cellCountArray' in initCellCountArray() in BoxCount.c.\n" );
    return;
  }

  // Initialise all values to zero. Note that, rather than use a 3D array ('int ***cellCountArray'), which is a bit
  // tedius to malloc() and free(), this is a simple 1D array ('int *cellCountArray'), so need to do the indexing 'by hand.'
  for( i=0; i<=nb; i++ )
    for( j=0; j<=nb; j++ )
      for( k=0; k<=nb; k++ )
        cellCountArray[ i*(nb+1)*(nb+1) + j*(nb+1) + k ] = 0;   // Convert 3D coordinates (i,j,k) to a 1D array index.
 }

// Counts all of the particles in each box. Also has the option to only count one cluster, but will count all clusters
// if onlyForCluster == -1. This essentially replaces the old OccElementCount().
void countAllParticles( int numBoxes, int r, int ***Positions, Cluster_T *ClusterList, int nClusters, int onlyForCluster )
{
  int ri, rj, rk, c;

  // Loop over all clusters.
  for( c=0; c<nClusters; c++ )
  {
    // If only considering one cluster, and this isn't it, skip to the next cluster.
    if( onlyForCluster!=-1 && c!=onlyForCluster ) continue;

    // Loop over all particles within this cluster.
    for( int nIndex=0; nIndex<ClusterList[c].NumParts; nIndex++ )
    {
      int p = ClusterList[c].Indices[nIndex];        // Shorthand for the particle index in the full "Positions" array.

      // Find the coordinates of the box that this particle belongs to. Note integer division rounds down.
      ri = (*Positions)[p][0] / r;
      rj = (*Positions)[p][1] / r;
      rk = (*Positions)[p][2] / r;

      // Sanity check: Make sure the box coordinates are valid. This check will slow things down a little, but
      // is probaby worth it for the extra peace of mind.
      if( ri<0 || ri>numBoxes || rj<0 || rj>numBoxes || rk<0 || rk>numBoxes )
      {
        printf( "ERROR in countAllParticles() in BoxCount: Bad box index.\n" );
        return;
      }

      // Increment the count of particles in this box. Note same indexing 'trick' as initialising cellCountArray.
      cellCountArray[ ri*(numBoxes+1)*(numBoxes+1) + rj*(numBoxes+1) + rk ]++;
    }
  }
}

// This simple function returns the number of particles in the given box. Really just converts from (i,j,k) coordinates
// to the array index.
int numParticlesInBox( int nb, int ri, int rj, int rk )
{
  // Sanity check on the provided box coordinates.
  if( ri<0 || ri>nb || rj<0 || rj>nb || rk<0 || rk>nb )
  {
    printf( "ERROR in numParticlesInBox() in BoxCount: Bad box index.\n" );
    return -1;
  }

  return cellCountArray[ ri*(nb+1)*(nb+1) + rj*(nb+1) + rk ];
}


//
// Call this routine to calculate and output to file the box count.
//
void WriteBoxCounting(int sysSize, int nClusters, int ***Positions, Cluster_T *ClusterList, int onlyForCluster){ //Runs box counting algorithm and writes output to DataOutput (there are multiple calls of DataToFile()).

  DataToFile(-1, 0); //calls function with these parameters. num defaults to 0 (not shown) as it is set in the function. since r = -1, just the header is printed on return of the fn.

  // If only one cluster is to be included in the counting, change this to the cluster index. Leave it at -1 to include all clusters.
  //int onlyForCluster = -1;
  

  // Vary box size r over the required range.
  for( int r=1; r<=sysSize; r++ )
  {
    int numBoxes = sysSize / r;     // Integer division, so rounds down, i.e. this is the number of boxes in each direction that do not intersect the bondary.
 
    // Initialise the array for counting all particles for this box size r.
    initCellCountArray( numBoxes );
 
    // Fill the count array with the numbers of particles in each. This replaces a lot of 'OccElementCount()'.
    countAllParticles( numBoxes, r, Positions, ClusterList, nClusters, onlyForCluster );

    // As per So, So and Jin, need the average number of "pixels" (occupied cells here) for boxes that do not intersect a boundary.
    int num_p_delta = 0;
    double p_delta = 0.0;

    for( int i=0; i<numBoxes; i++ )     // Note these loops are over boxes that are fully within the system box, not intersecting the boundaries.
      for( int j=0; j<numBoxes; j++ )
        for( int k=0; k<numBoxes; k++ )
        {
          int numInThisBox = numParticlesInBox(numBoxes,i,j,k);

          if( numInThisBox>0 )
          {
            p_delta += numInThisBox;
            num_p_delta++;          // Note p_delta is the average of blocks "which are counted as 1", as per the article.
          }
        }

    if( num_p_delta ) p_delta /= num_p_delta;      // Now gives the mean "pixel" for counted boxes, p_{\delta} in the article.

    // Change to So, So and Jin: Take p_delta to be the _fraction_ of occupied cells rather than the number,
    // to avoid "double weighting" by volume for those boxes that intersect the boundary.
    p_delta /= r*r*r;

    // Get the total weighted box occupancy as per equation (7) in the So, So and Jin article.
    double Nr = 0.0;

    for( int i=0; i<=numBoxes; i++ )         // Notice these loops include numBoxes ("<=" rather than just "<"), so we are including boxes that intersect with the boundary.
      for( int j=0; j<=numBoxes; j++ )
        for( int k=0; k<=numBoxes; k++ )
        {
          // The second term in the equation is the fraction of the box that was counted over, i.e. the volume of the "shortened" box to
          // the volume r*r*r for boxes that do not intersect the boundary.
          double volRatio = 1.0;
          if( r*(i+1)-1 >= sysSize ) volRatio *= 1.0*(sysSize-r*i)/r;      // i.e. if upper limit exceeded system size, reduce weighting proportionally.
          if( r*(j+1)-1 >= sysSize ) volRatio *= 1.0*(sysSize-r*j)/r;      // Same for the other two axes.
          if( r*(k+1)-1 >= sysSize ) volRatio *= 1.0*(sysSize-r*k)/r;

		  if(!volRatio) continue; //if volRatio is zero after it has been calculated, don't continue as there is a potential division-by-zero later on.

          // This is the same p_ij as in the So, So and Jin article.
          double p_ij = numParticlesInBox( numBoxes, i, j, k );

          // Change to So, So and Jin: Take p_ij to be the fraction of cells, rather than the absolute number.
          // This means dividing by the volume of the box, taking into account it will be less than r*r*r if
          // it intersects the boundary. However, have (almost) just calculated this anyway.
          double V_ij = r*r*r * volRatio;

          p_ij /= V_ij;     // p_i is now a fraction, rather than a number.

          // Now continue as per So, So and Jin, with p_delta and p_ij fractions rather than counts.
          Nr += ( p_ij<p_delta ? p_ij/p_delta : 1.0 ) * volRatio;
        }

    // Output a table row for this value of r.
    DataToFile( r, Nr );

    // Free the counting array for this box size r.
    free( cellCountArray );
  }

  DataToFile(0, 0); // add new line after writing file.close starting for loop, one of three
}






//helper_function(); //call this to help implement this fn?
//}
/*Will include BoxCounting.h in main DLA programme and then call BoxCounting function above in 'main' code to implement the modular programme of box counting algorithm */

//
// OLD OccElementCount() - worked but was slow.
//
// Returns the number of cells occupied by a particle in the r*r*r box with the given coordinates for one corner.
// The optional argument "onlyForCluster" only performs the calculation over the requested cluster.
// If this argument is not specified or is -1, include all clusters in the calculation.
// //
// int OccElementCount(int r, int i, int j, int k, int ***Positions, Cluster_T *ClusterList, int sysSize, int onlyForCluster ){ // This fn detects, for a given box size, the number of boxes containing an occupied element.

//   // Loop over all cells within this r*r*r box and count how many are occupied.
//   int numOccupied = 0;

//   for (int l=0; l<r; l++) {
//     for (int m=0; m<r; m++) {
//       for (int n=0; n<r; n++) {

//         // Do not consider cells that lie outside the primary copy of the simulation box; in other words,
//         // those for which we would normally apply the periodic boundary conditions.
//         if( i+l>=sysSize || j+m>=sysSize || k+n>=sysSize ) continue;

//         // Loop over all clusters.
//         for( int c=0; c<NUMPARTICLES; c++ )
//         {
//           // If only considering one cluster, and this isn't it, skip to the next cluster.
//           if( onlyForCluster!=-1 && c!=onlyForCluster ) continue;

//           // Loop over all particles within this cluster, counting those within this box.
//           for( int nIndex=0; nIndex<ClusterList[c].NumParts; nIndex++ )
//           {
//             int p = ClusterList[c].Indices[nIndex];        // Shorthand for the particle index in the full "Positions" array.

//             if( (*Positions)[p][0]==i+l && (*Positions)[p][1]==j+m && (*Positions)[p][2]==k+n ) numOccupied++;
//               //i,j,k are coords for the corner of the box and l,m,n are the relative coords within the box (always therefore less than r). So actual coordinates for a particle are i+l, j+n, etc. 
//           }
//         }
 
//       } // End of 'n' loop.
//     } // End of 'm' loop.
//   } // End of 'l' loop.

//   return numOccupied;
// }

