#include <stdio.h>
#include <stdlib.h> //additional inclusion of libraries, rand()(and later srand()) for generating random numbers.
#include <time.h> //time is a function for reading the current time, stored in an external library.
#include <stdbool.h>
#include <string.h> //to allow declaration of built-in fn 'strlen'
#include "BoxCount.h" //box counting algorithm function will be called from this header file (programme module).
#include "VariableParameters.h" //the file containing variable parameters for the system which users can change before recompiling the main code.
#include "Cluster_T_Struct.h"
#include <math.h> //for using exponent function.
#include "Percolation.h" //module which identifies if cluster is percolating or not.

/********************************************************Functions+objects*********************************************************************************/

void SavePositionsFileName(int **Positions, int SystemSize, int NumParticles, const char *FileName, Cluster_T **ClusterList){  // fn for saving the particles' position coords in 3D to an external file in csv format. For 3d visual display. 2** for Positions as it is a 2d array (pointer to pointer), but don't need extra pointer (no need to pass by ref) as this fn does not change where the pointer points in memory.
  FILE *PositionsOutput; //FILE is a structure which must be pointed to
  PositionsOutput = fopen(FileName, "w"); //write to and open a new file called ParticlePosition.csv. Beware this writes over any file with this name in the directory! "a" appends data on new run to file (with a new line space)

  if (!PositionsOutput){
    printf("\nClose open .csv file to append to it!");
    return;
  }
  
  
  fprintf (PositionsOutput,"System dimensions in 3D called N, %i \n", SystemSize);
  fprintf (PositionsOutput,"Number of seeds in system, %i \n", NumParticles);
  fprintf (PositionsOutput,"\n"); //adds a new line to separate appended data to follow
  
  for(int i=0; i<NumParticles; i++) //index i for label/distinguishing each particle
		fprintf(PositionsOutput, "%i, %i, %i \n", Positions[i][0], Positions[i][1], Positions[i][2]); //Prints the coordinate in each plane (x, y and z) for every particle. '(*Positions)' means pointer to the whole 2D array ('Positions') of elements
  
  fprintf (PositionsOutput,"\n"); //adds a new line to separate appended data to follow  
  
  // Loop through all clusters.
  for(int i=0; i<NumParticles/SEED_LENGTH; i++)		//where there are seeds of size SEED_LENGTH, need to work through reduced number of initial particles in the box
  { 
	  	
  	if((*ClusterList)[i].NumParts == 0) //don't print out empty clusters in list
			continue;
   	   
    		for(int j=0; j<(*ClusterList)[i].NumParts; j++) //loop through the particles in the cluster
      			fprintf(PositionsOutput, "%i ", (*ClusterList)[i].Indices[j]); //Listing the indices of the particles in the cluster
      	    		
    	fprintf(PositionsOutput, "\n");// start newline after printing index of cluster and indices of particles in that cluster.
  	}
  
  
  fclose(PositionsOutput); //close file pointed to to avoid loss of data or runtime errors
  
}


void MultiSavePositionsFileName(int **Positions, int SystemSize, int NumParticles, int Count, Cluster_T **ClusterList){  // fn for saving the particles' position coords in 3D to an external file in csv format. For 3d visual display. 2** for Positions as it is a 2d array (pointer to pointer), but don't need extra pointer (no need to pass by ref) as this fn does not change where the pointer points in memory.

  char buf[100]; //max of 100 characters for file name
  snprintf(buf, sizeof(buf), "Positions_%04d.csv", Count); //print to buffer. Allows file name to be stored in a string to be used in fopen next.
  
  SavePositionsFileName(Positions, SystemSize, NumParticles, buf, ClusterList);
  
}


void GelPointPositionsFileName(int **Positions, int SystemSize, int NumParticles, int Count, Cluster_T **ClusterList){  // fn for saving the particles' position coords in 3D to an external file in csv format. For 3d visual display. 2** for Positions as it is a 2d array (pointer to pointer), but don't need extra pointer (no need to pass by ref) as this fn does not change where the pointer points in memory.

  char buf[100]; //max of 100 characters for file name
  snprintf(buf, sizeof(buf), "GelPoint_T%05d.csv", Count); //print to buffer. Allows file name to be stored in a string to be used in fopen next.
  
  SavePositionsFileName(Positions, SystemSize, NumParticles, buf, ClusterList);
  
}



void MallocParticles(int ***Positions, int NumParticles){  //fn for allocating the memory for the indices and coordinates for all individual particles in the system. *** since Positions is already a pointer to a pointer (as a 2d array) and we want to its address in memory! Passing as a pointer to a pointer to a pointer...

//First index in an array for the positions of particles - first index is particle labels/indices. Allocate this memory.
  (*Positions) = (int **)malloc(sizeof(int*)*NumParticles); //size of int* tells the number of bytes a single int * is. times by NUMPARTICLES (in variable parameters header file) for number of rows. malloc automatically refers to void, so must point it to int.  int ** (the 'cast') to match up with **Positions. Since memory has been allocated, the variables of Positions are not automatically removed from memory (they are not local variables in the stack but rather dynamically allocated variables in the heap that need to be actively removed). Need to remove the variable pointed at by the pointer. A pointer is a variable that contains the address of another variable.
	
	//Second index of the array, want to be 0, 1 or 2 for each of x, y and z coordinates - 2nd index is particle coordinates. Allocate this memory.
  for(int i=0; i<NUMPARTICLES; i++)
	  (*Positions)[i]= (int*)malloc(sizeof(int)*3); //3 for the 3 x y and z dimensions. (int *) is a 'cast' - where the malloc function returns the address (pointer to) the first byte. By default, malloc returns a pointer to void (void *), so we must cast it to int*. 
  //(*Positions)[i] rather than (*Positions[i]) otherwise, it tries to get the array index first before converting the reference back to an actual value, causing a seg fault.
}



void MallocClusters(Cluster_T **ClusterList, int nClusters){  //fn for allocating the memory for (max) number of clusters in system and their indices. ** since ClusterList is a structure which is a pointer and we want to point to this! Passing as a pointer to a pointer.

  //allocate memory for (max) number of clusters:
  	*ClusterList = (Cluster_T*)malloc(sizeof(Cluster_T)*nClusters); //memory, in bytes, automatically allocated by compiler for each cluster. Multiplied by number of particles/initial clusters (seeds of non-unitary size) in system for maximum number of clusters where each is (initially) of size one.

	/* All clusters are initially SEED_LENGTHx1 rodlike 'seeds', where SEED_LENGTH can be 1. */
	for(int i=0; i<nClusters; i++) 
		(*ClusterList)[i].Indices = (int*)malloc(sizeof(int)*SEED_LENGTH);			// SEED_LENGTH particles per cluster.
}



void PBCs(int* X_Ptr, int* Y_Ptr, int *Z_Ptr, int SystemSize){ //fn for applying periodic boundary conditions (wrap-around). 'int' means pointer points to an integer value.
	
	if (*X_Ptr>=SystemSize) //if the value the pointer, *X_Ptr, points to (dereferencing the value) is greater than SystemSize, ...
		*X_Ptr-=SystemSize; //If cluster is on a boundary on right, wrap it round to the other side (*X_Ptr = *X_Ptr - SystemSize)
	if (*X_Ptr<0)
		*X_Ptr+=SystemSize;
	if (*Y_Ptr>=SystemSize) //and for other planes
		*Y_Ptr-=SystemSize; 
	if (*Y_Ptr<0)
		*Y_Ptr+=SystemSize;
	if (*Z_Ptr>=SystemSize)
		*Z_Ptr-=SystemSize; 
	if (*Z_Ptr<0)
		*Z_Ptr+=SystemSize; 
				
}


void InitialisePositions(int **Positions ,int NumParticles, int nClusters, int N){ //N is system size, declared in main.

	/*SEED_LENGTH x1 seeds. */
	for( int c=0; c<nClusters; c++ )
	{
		int good;

		do
		{
			good = 1;		// Initialise 'good==true' (1) and set to 'false (0)' if the trial cluster is 'bad', i.e. overlaps another cluster.

			// The start particle index for this cluster. The final one is (c+1)*SEED_LENGTH - 1 (inclusive).
			int startIndex = c * SEED_LENGTH;

			// Initialise position of first particle in this cluster.
			for( int k=0; k<3; k++ ) Positions[startIndex][k] = rand()%N;

			// If overlapping another particle, re-try ('break' exits the 'for' loop to save time).
			for( int p=0; p<startIndex; p++ )
				if( Positions[startIndex][0]==Positions[p][0] && Positions[startIndex][1]==Positions[p][1] && Positions[startIndex][2]==Positions[p][2] )
				{
					good = 0;
					break;
				}

			// If particle 1 was bad, try again.
			if( good==0 ) continue;

			// Choose a random direction as an integer from 1 to 6 (could get away with 1 to 3?).
			int dirn = rand()%6;
			
			// Loop through all remaining particles in this seed, not including the first. If SEED_LENGTH==1, this loop will be automatically skipped.
			for( int i=1; i<SEED_LENGTH; i++ )
			{
				// Start at same position as the previous particle.
				for( int k=0; k<3; k++ ) Positions[startIndex+i][k] = Positions[startIndex+i-1][k];

				// Move up/down/left/right etc. by one. This is to plant the second particle of this seed. Note the direction is fixed within this loop.
				switch( dirn )
				{
					case 0	:	Positions[startIndex+i][0]++; break;
					case 1	:	Positions[startIndex+i][0]--; break;
					case 2	:	Positions[startIndex+i][1]++; break;
					case 3	:	Positions[startIndex+i][1]--; break;
					case 4	:	Positions[startIndex+i][2]++; break;
					case 5	:	Positions[startIndex+i][2]--; break;

					default:
						printf( "ERROR in InitialisePositions with SEED_LENGTHx1 seeds: bad direction %i.\n", dirn );		// Sanity check. Should not be able to happen.
				}

			// Wrap particle to to the periodic boundaries (no need to do this for particle 1 as it is definitely in the primary cell).
			PBCs( &Positions[startIndex+i][0], &Positions[startIndex+i][1], &Positions[startIndex+i][2], N );

			// Check article i is not overlapping any previously-placed particles.
			for( int p=0; p<startIndex+i; p++ )
				if( Positions[startIndex+i][0]==Positions[p][0] && Positions[startIndex+i][1]==Positions[p][1] && Positions[startIndex+i][2]==Positions[p][2] )
				{
					good = 0;
					break;
				}		

			}		// End of i-loop (particles within the cluster).

		} while( good==0 );
		// If reached here, cluster was 'good' in the sense of not overlapping other clusters; move onto the next cluster.
	}
}

















void PrintParticlePositions(int ***Positions, int NumParticles){ //print particle labels and positions to terminal

	for(int i=0; i<NumParticles; i++) //i is particle label. Loop through them. 
		printf("Index= %i with initial coordinates %i, %i, %i \n", i, (*Positions)[i][0], (*Positions)[i][1], (*Positions)[i][2]); //test to see array of random 3D positions of particles in terminal.
}



void PrintClusterInfo(Cluster_T **ClusterList, int nClusters){//print labels for clusters and the indices for the particles in each cluster as well as checking for errors.

	int RunningTotal = 0; //Running total of particles in clusters
	
	for(int i=0; i<nClusters; i++){ //i is cluster index (also equivilent to number of particles in system)
		if((*ClusterList)[i].NumParts == 0) //don't print out empty clusters in list
			continue;
   		printf("Index of cluster = %i\t", i); //print labels for each cluster 
   
    		for(int j=0; j<(*ClusterList)[i].NumParts; j++) //loop through the particles in the cluster
      			printf("%i ", (*ClusterList)[i].Indices[j]); //Listing the indices of the particles in the cluster
      		RunningTotal += (*ClusterList)[i].NumParts;	//add up all number of particles in all clusters
    		
    	printf("\n");// start newline after printing index of cluster and indices of particles in that cluster.
  	}
	#ifdef SEED_LENGTH
  	if (RunningTotal/SEED_LENGTH != nClusters)
  		printf("Error: particle number in clusters different to initial."); // basic error check to see if number of particles at the end of the simulation (in clusters) is the same as at the beginning

	#else
	if (RunningTotal != nClusters)
  		printf("Error: particle number in clusters different to initial."); // basic error check to see if number of particles at the end of the simulation (in clusters) is the same as at the beginning
	
	#endif

}


void AllClusterSizes(const char *FileName, Cluster_T **ClusterList, int NumClusters){//print labels for clusters and the indices for the particles in each cluster as well as checking for errors.
	
	FILE *AllClustersOutput; //FILE is a structure which must be pointed to
	AllClustersOutput = fopen(FileName, "w"); //write to and open a new file called ParticlePosition.csv. Beware this writes over any file with this name in the directory! "a" appends data on new run to file (with a new line space)

	if (!AllClustersOutput){
		printf("\nClose open .csv file to append to it!");
		return;
	}
	
	fprintf (AllClustersOutput,"Index,");
	fprintf (AllClustersOutput,"Monomer mass");
	fprintf (AllClustersOutput,"\n"); //adds a new line to separate appended data to follow
	
	
	for(int i=0; i<NumClusters; i++){ //i is cluster index (also equivilent to number of particles in system)
		int RunningTotal = 0; //Start running total for number of particles in a cluster. 
	
		if((*ClusterList)[i].NumParts == 0) //don't print out empty clusters in list
			continue;
			
		fprintf(AllClustersOutput, "%i, ", i); //print labels for each cluster to file
   	   
    	for(int j=0; j<(*ClusterList)[i].NumParts; j++) //loop through the particles in the cluster
			RunningTotal += 1;	//add up number of particles in this cluster.
      		//fprintf(sizeof, (*ClusterList)[i].Indices[j]); //Listing the indices of the particles in the cluster
				
      	fprintf(AllClustersOutput, "%i ", RunningTotal); //print the number of particles in this cluster to file. 
		fprintf(AllClustersOutput, "\n");// start newline after printing index of cluster and number of particles in that cluster.
		
		//RunningTotal = 0; //reset the running total of particles for the next cluster. 
  	}
  
  	fclose(AllClustersOutput); //close file pointed to to avoid loss of data or runtime errors
  	
}




void CheckParticleIndices(Cluster_T **ClusterList, int NumParticles, int nClusters) { //check each particle index appears exactly once in the system.

	int i=0;
		
	for(i=0; i<nClusters; i++){ //loop through all particle indices. 
		int RepeatedParticleIndex = 0;
		for(int j=0; j<nClusters; j++) //loop through all cluster indices (which is equivilent to number of particles in system).
			for(int k=0; k<(*ClusterList)[j].NumParts; k++) //loop through particles in clusters
				if ((*ClusterList)[j].Indices[k] == i)
					RepeatedParticleIndex++; //Count up each time a particle with this index appears. Should only be once.
		if(RepeatedParticleIndex!=1)
			printf("Particle index %i occured %i times\n", i, RepeatedParticleIndex);			
				
	}
			
}


void CheckOverlapping(int **Positions, int NumParticles){ //check if any particles are overlapping and thus sizes undercounted.
	for(int i=0; i<NumParticles; i++){ //loop through all particle indices.
		for(int j=0; j<NumParticles; j++){ //loop through all other particle indices (check for overlapping).
			if(i<=j) continue; //skip over same pairs of indices
			if(Positions[i][0]==Positions[j][0] && Positions[i][1]==Positions[j][1] && Positions[i][2]==Positions[j][2]){ //i is current trialling particle, checking against exisiting particles, j (looping through js)
				printf("Error: Particles overlapping\n");
				return;
			}
		}
	}
}



void InitialiseClusterList(Cluster_T **ClusterList, int nClusters){ //initialise the members of the ClusterList structure.

	for( int i=0; i<nClusters; i++ )
	{
		(*ClusterList)[i].NumParts = SEED_LENGTH;			// SEED_LENGTH monomers in each initial cluster.

		// Particle indices in each seed are consecutive.
		for( int j=0; j<SEED_LENGTH; j++ )
			(*ClusterList)[i].Indices[j] = SEED_LENGTH*i + j;
	}
}



void FreeMemory(int ***Positions, Cluster_T **ClusterList, int NumParticles, int nClusters){ //fn that frees allocated memory for both the individual particles and clusters (all memory allocated).

 for(int i=0; i<NumParticles; i++) free((*Positions)[i]); //free up memory allocated for the x, y and z coordinates in the array for each particle.
	free ((*Positions)); //freeing up number of bytes per row pointer (reverse order freeing up memory). Freeing up the labels for the particle positions
     
    for(int i=0; i<nClusters; i++) 
      if ((*ClusterList)[i].Indices != NULL) //check if there is anything to free
          free((*ClusterList)[i].Indices); //free up memory allocated for the labels of the clusters in the list.
          
    free((*ClusterList)); //free memory for the list itself.
}




float RandNum(){ //function for returning a random number between 0 and 1

	float p;
	
	do{	
		int n = rand();  //n is an integer number in the range 0 to RAND_MAX -1 (what rand() returns)
		p = ((float)n/RAND_MAX); //(float)n is a function to locally convert n to a float so as not to have integer division which will always return 0 for p (probability) being a number between 0 and 1. 
		
	}while(p<0 || p>=1); //To avoid seg fault if randnum generator returns number below 0 or above (or equal to) 1.
	return p;	
}





void TrialMove(Cluster_T **ClusterList, int nClusters, int SelectedLabel, int ***Positions, int delta_x, int delta_y, int delta_z){//function to move cluster, if the move is allowed.
	int Good = 1; 	//will move cluster if good = 1.
	
	if ((*ClusterList)[SelectedLabel].NumParts == 0) return; //If the cluster we are trying to move is of zero size, don't try to move it (sim efficiency).'return' to jump out of routine	(as if at end of routine)	
			
	for(int i=0; i<nClusters; i++){ //for all the clusters in the list (out of max number which is equal to number of particles in system)
		if(i == SelectedLabel) continue; //if selected cluster (index) is the one program is trying to move, skip over it - we want to check against other clusters.
		for (int j = 0; j<(*ClusterList)[i].NumParts; j++) //loop over all particles (j) in the cluster (i) we are trying to check (if we are able to move to).
			for(int k = 0; k<(*ClusterList)[SelectedLabel].NumParts; k++){ //SelectedLabel is the index for the cluster we are trying to move (looking at all the particles (k) in the cluster)
				int P1 = (*ClusterList)[i].Indices[j]; 	// P1 is global seed/particle index for seed/particle j in cluster i (the cluster we are checking against)
				int P2 = (*ClusterList)[SelectedLabel].Indices[k]; //Same but for seed/particle k in cluster 'SelectedLabel' - the cluster we are trying to move.
				
				int TrialX =(*Positions)[P2][0]+ delta_x ; //TrialX is the x position the particle (in the cluster we are trying to move) has. It will wrap round to other side of system if on border (PBCs)
				//printf("Original TrialX = %i\n", TrialX);
				int TrialY =(*Positions)[P2][1]+ delta_y ;
				int TrialZ =(*Positions)[P2][2]+ delta_z ;
								
				PBCs(&TrialX, &TrialY, &TrialZ, SYSTEMSIZE); //Apply PBCs for all planes before checking if clusters will overlap
				
				//printf("New TrialX = %i\n", TrialX);
				if(TrialX ==(*Positions)[P1][0] && TrialY ==(*Positions)[P1][1] && TrialZ ==(*Positions)[P1][2]){ //checks if particles in each cluster (moving and possibly to move onto) will overlap on moving (e.g. by delta_x). This is after PBCs have been applied
					Good = 0; //If a particle in the cluster being checked is already occupying the position, this position is not 'good' to move the cluster i to.
					break;
				}
							
				

			}
	}
	
	if (Good == 1) { //now do this, if we are able to move the cluster without overlapping another cluster. Update the position of the cluster by deltax, y and z. Check if, after updating the position of the cluster, the particle is on a boundary. If so, wrap it to other side.
		for(int k = 0; k<(*ClusterList)[SelectedLabel].NumParts; k++){ 	//for particle k in the cluster we are moving
			int P2 = (*ClusterList)[SelectedLabel].Indices[k]; //have to redefine p2 here as before since in a new loop (local variable)
						
			(*Positions)[P2][0] += delta_x; //Update the position of the labelled cluster in the x direction (even if that change in x is 0). All particles in the cluster will have moved by delta x after looping all ks. Gives collective effect of moving whole cluster by end of loop.									
			(*Positions)[P2][1] += delta_y;		
			(*Positions)[P2][2] += delta_z;
				
			PBCs(&(*Positions)[P2][0], &(*Positions)[P2][1], &(*Positions)[P2][2], SYSTEMSIZE); //Apply PBCs function for newly moved cluster.
			
			
		}
	}
	
	
}


float MassProb(Cluster_T **ClusterList, int SelectedLabel, float FractalDim){ //Probability for moving clusters, dependent on their mass. Large clusters move less freq (mimicking slow diffusion of real large objects) 
	int M_i = (*ClusterList)[SelectedLabel].NumParts; //Mass of the cluster being moved (in number of particles). 
	float alpha = (-1.0/FractalDim);	//2.4 is trial estimated fractal dimension of final aggregate.
	float P_i = pow(M_i, alpha); //function for giving exponent below. first is base, 2nd argument is exponent.
					//p_i is probability of (randomly selected) cluster moving = cluster i of M_i monomers (monomers of unit mass so cluster mass is equal to #monomers) (ref milk gelation paper) to power of positive mobility exponent (ref 'transition between floc...' - Gimel)
	return P_i;
}


int ClustersAdj(Cluster_T **ClusterList, int Cluster1, int Cluster2, int ***Positions){ //Function to check if sites of two clusters are adjacent. int return will be the number of adjacent sites to use later on (e.g. if many sites are adj, those clusters could aggregate with a higher probability)
	
	int AdjX, AdjY, AdjZ;
	int NumAdj = 0;//Count the number of adjacent particles in the two clusters
									//Cluster 1 and 2 for checking if two clusters are adjacent.
	for (int j = 0; j<(*ClusterList)[Cluster1].NumParts; j++) //loop over all particles in the cluster1 
		for(int k = 0; k<(*ClusterList)[Cluster2].NumParts; k++){ 	//Cluster2 is the index for the cluster we are checking if adj (looking at all the particles (k) in the cluster)
			int P1 = (*ClusterList)[Cluster1].Indices[j]; 	// P1 is global particle index for particle j in cluster1 (the cluster we are checking against)
			int P2 = (*ClusterList)[Cluster2].Indices[k]; //Same but for particle k in cluster2
		
			AdjX =(*Positions)[P1][0]+ 1 ; //Looking at cluster possibly to the right of another
			AdjY =(*Positions)[P1][1]+ 0 ;
			AdjZ =(*Positions)[P1][2]+ 0 ;
			
			PBCs(&AdjX, &AdjY, &AdjZ, SYSTEMSIZE); //wrap around system if on edge before checking if clusters are adjacent.
			
			if(AdjX ==(*Positions)[P2][0] && AdjY ==(*Positions)[P2][1] && AdjZ ==(*Positions)[P2][2]) //checks if particles in each cluster (moving and possibly to move onto) will overlap on moving (e.g. delta_x)
				NumAdj++; //If particles in the clusters are adjacent, add these to the count. Continue adding until all adjacent particles on this side are counted.
			
			
			AdjX =(*Positions)[P1][0]- 1 ; //Now looking at cluster possibly to the left of another
			AdjY =(*Positions)[P1][1]+ 0 ;
			AdjZ =(*Positions)[P1][2]+ 0 ;
			
			PBCs(&AdjX, &AdjY, &AdjZ, SYSTEMSIZE); //wrap around system if on edge
			
			if(AdjX ==(*Positions)[P2][0] && AdjY ==(*Positions)[P2][1] && AdjZ ==(*Positions)[P2][2]) //checks if particles in each cluster (moving and possibly to move onto) will overlap on moving (e.g. delta_x)
				NumAdj++; //If particles in the clusters are adjacent, add these to the count. Continue adding until all adjacent particles on this side are counted.
			
			
			AdjX =(*Positions)[P1][0]+ 0 ; 
			AdjY =(*Positions)[P1][1]+ 1 ; 
			AdjZ =(*Positions)[P1][2]+ 0 ;
			
			PBCs(&AdjX, &AdjY, &AdjZ, SYSTEMSIZE); //wrap around system if on edge
			
			if(AdjX ==(*Positions)[P2][0] && AdjY ==(*Positions)[P2][1] && AdjZ ==(*Positions)[P2][2]) //checks if particles in each cluster (moving and possibly to move onto) will overlap on moving (e.g. delta_x)
				NumAdj++; //If particles in the clusters are adjacent, add these to the count. Continue adding until all adjacent particles on this side are counted.
			
			
			AdjX =(*Positions)[P1][0]+ 0 ; 
			AdjY =(*Positions)[P1][1]- 1 ;
			AdjZ =(*Positions)[P1][2]+ 0 ;
			
			PBCs(&AdjX, &AdjY, &AdjZ, SYSTEMSIZE); //wrap around system if on edge
			
			if(AdjX ==(*Positions)[P2][0] && AdjY ==(*Positions)[P2][1] && AdjZ ==(*Positions)[P2][2]) //checks if particles in each cluster (moving and possibly to move onto) will overlap on moving (e.g. delta_x)
				NumAdj++; //If particles in the clusters are adjacent, add these to the count. Continue adding until all adjacent particles on this side are counted.
			
			
			AdjX =(*Positions)[P1][0]+ 0 ; 
			AdjY =(*Positions)[P1][1]+ 0 ;
			AdjZ =(*Positions)[P1][2]+ 1 ; 
			
			PBCs(&AdjX, &AdjY, &AdjZ, SYSTEMSIZE); //wrap around system if on edge
			
			if(AdjX ==(*Positions)[P2][0] && AdjY ==(*Positions)[P2][1] && AdjZ ==(*Positions)[P2][2]) //checks if particles in each cluster (moving and possibly to move onto) will overlap on moving (e.g. delta_x)
				NumAdj++; //If particles in the clusters are adjacent, add these to the count. Continue adding until all adjacent particles on this side are counted.
			
			
			AdjX =(*Positions)[P1][0]+ 0 ; 
			AdjY =(*Positions)[P1][1]+ 0 ;
			AdjZ =(*Positions)[P1][2]- 1 ;
			
			PBCs(&AdjX, &AdjY, &AdjZ, SYSTEMSIZE); //wrap around system if on edge
			
			if(AdjX ==(*Positions)[P2][0] && AdjY ==(*Positions)[P2][1] && AdjZ ==(*Positions)[P2][2]) //checks if particles in each cluster (moving and possibly to move onto) will overlap on moving (e.g. delta_x)
				NumAdj++; //If particles in the clusters are adjacent, add these to the count. Continue adding until all adjacent particles on this side are counted.
			
	
		}

return NumAdj; //return the number of adjacent sites between the two clusters being checked.
}


void AggregateClusters(Cluster_T **ClusterList, int Cluster1, int Cluster2){//update cluster lists for merging clusters - called after recognition of clusters being adjacent.
	int NumParts1 = (*ClusterList)[Cluster1].NumParts; //Number of particles in cluster 1
	int NumParts2 = (*ClusterList)[Cluster2].NumParts; //Number of particles in cluster 2
	int *Indices1 = (*ClusterList)[Cluster1].Indices; //Indices of particles in cluster 1 (pointer to first element in 1d array of list of indices)
	int *Indices2 = (*ClusterList)[Cluster2].Indices; //Indices of particles in cluster 2
	
  int *NewIndices1 = (int *)malloc(sizeof(int)*(NumParts1+NumParts2)); //update list of particle indices for Cluster1 after particle aggregation (adding cluster 2 particles).
 
  for(int i =0; i<NumParts1; i++){	//counting through particles in cluster1
    NewIndices1[i] = Indices1[i]; //Copying over Cluster1 Indices1 to new Cluster1 indices as its number updates with aggregation.
  }
 
	for(int i =0; i<NumParts2; i++){	//counting through particles in cluster2
		NewIndices1[NumParts1+i] = Indices2[i]; //changing the array of Newcluster1 indices. Adding all particles making up (adj) cluster2 indices to the end of the new cluster1 indices list.
	
	}
	
  free(Indices1); //free up old memory of cluster1
  (*ClusterList)[Cluster1].Indices = NewIndices1; //Rest of code will use the new indices (equivalent to Indices1) for cluster 1 as it is updated.
	(*ClusterList)[Cluster1].NumParts += NumParts2; //update Cluster1 num of particles with the number of particles transferred from cluster2.
	(*ClusterList)[Cluster2].NumParts = 0; //Cluster2 now no longer exists (is merged with 1)
  free(Indices2); //free memory of the Cluster2 entry that now no longer exists.
  (*ClusterList)[Cluster2].Indices = NULL; // This sets the permanent pointer (not local copy) to the indices to NULL so as not to try free memory again - says memory is empty.
}



void SystemSizeCheck(int SystemSize, int NumParticles){ //check that the number of particles chosen can fit in the chosen system size, if not abort.
	
	if ((SystemSize*SystemSize*SystemSize)<NumParticles){
		printf("Increase system size or decrease number of particles!");
		exit(0); //terminate programme.
	}
			
}





int MeanClustSizes(Cluster_T **ClusterList, int nClusters){ //fn to find the mean size of clusters (in # particles) when called at any point in the simulation. Also largest cluster size.

	int NumClusters = 0; //initialise number of clusters.
	int i;
	

	for(i=0; i<nClusters; i++){ //i is cluster index (also equivilent to number of seeds in system)
				
		if((*ClusterList)[i].NumParts == 0) //ignore empty clusters in list
			continue;
   	//	printf("Index of cluster = %i\t", i); //print labels for each cluster 
   
    	//   	printf("Size %i\n", (*ClusterList)[i].NumParts); //prints the size of the cluster at this point.
    	   	NumClusters ++; //add 1 to the count of number of clusters
      		
	
	}
	//printf("Number of clusters = %i\n", NumClusters); //total number of clusters counted by end of this run.
	float Average = (float)nClusters/(float)NumClusters; //average size of clusters in units of particle number
	//printf("Average cluster size = %.2f\n", Average);
	return Average;
}




int LargestClustSize(Cluster_T **ClusterList, int nClusters){ //fn to find the mean size of clusters (in # particles) when called at any point in the simulation. Also largest cluster size.

	int MaxSize = 0; //initialise maximum size of cluster (so far).
	int i;
	

	for(i=0; i<nClusters; i++){ //i is cluster index (also equivilent to number of particles in system)
				
   		if((*ClusterList)[i].NumParts>MaxSize)
			
			// Output the number of initial rod-like seeds in the largest cluster (not the number of monomers).
   			MaxSize = ((*ClusterList)[i].NumParts)/SEED_LENGTH; //Every time a cluster is larger than the one just checked, it is asigned as the largest cluster. by end of loop, largest cluster size is given.  	
	}
	//printf("Max size %i\n", MaxSize); //prints the size of the largest cluster at this point.
	return MaxSize;
}


int PercolClustSize(Cluster_T **ClusterList, int nClusters, int ***Positions){ //fn to find the mean size of clusters (in # particles) when called at any point in the simulation. Also largest cluster size.


	int PercolCluster = IndexOfPercolatingCluster(nClusters, *ClusterList, *Positions); //returns index of percolating cluster, or -1 if there are none. 
	int PercolSize = 0; //initialise size of percolating cluster (so far).
	
	PercolSize = ((*ClusterList)[PercolCluster].NumParts)/SEED_LENGTH; // the size of the percolating cluster, with index 'PercolCluster'. 
 	
	//printf("Max size %i\n", MaxSize); //prints the size of the largest cluster at this point.
	return PercolSize;
} 


void SizesToFile (int T, int Mean, int Largest, int PercolSize){ //fn for writing the mean and largest cluster size data to an external file. Called in 'WriteSizesToFile'
  FILE *DataOutput; //FILE is a structure which must be pointed to
  DataOutput = fopen("ClusterSizes.csv", "a"); //write to and open a new file called ClusterSizes. Beware this writes over any file with this name in the directory! "a" appends data on new run to file (with a new line space)
  if (!DataOutput){
    printf("Close open .csv file first!");
    return;
  }

	if (T==0){
		fprintf(DataOutput, "T, Mean, Largest, PercolSize\n"); //to print header at start of file. It will do this on the first loop of a run of the programme, as this is the initial condition in called function.
		fprintf(DataOutput, "%i, %i, %i, %i\n", T, Mean, Largest, PercolSize); // for writing to a .csv file. Prints first time interval at T =0
	}
	else if (T==-1)
		fprintf(DataOutput, "\n"); //add new line at end of file (to separate appended data).
	else 
  		fprintf(DataOutput, "%i, %i, %i, %i\n", T, Mean, Largest, PercolSize); // for writing to a .csv file: 
  
  fclose(DataOutput); //close file pointed to to avoid loss of data or runtime errors

}
























void WriteSizesToFile (int T, Cluster_T *ClusterList, int nClusters, int **Positions, int N){ //write cluster size quantities to file in given time intervals

	
	if(T>-1 && T<60000){ //print .csv files more frequently to begin with at the start
		if ((int)T % 5 == 0){ //if, when dividing the new T by 100, the remainder is 0 (i.e. T is a multiple of 100), print the particle positions to file (snapshots)
			printf("T= %05d\n",(int)T); //04 meaning the number is 4 digits long and is padded with zeros to ensure this. /r is carriage return to write back over line
			

			int Mean = MeanClustSizes(&ClusterList, nClusters); //call fn to show mean cluster sizes at these intervals and set equal to 'mean'
			int Largest = LargestClustSize(&ClusterList, nClusters);
			
			int Percol = PercolClustSize(&ClusterList, nClusters, &Positions); //the number of particles in the cluster whose index is assigned for the percolating cluster.

			
			SizesToFile(T, Mean, Largest, Percol);
			
			/*Comment in for interval position print out*/
			//MultiSavePositionsFileName(Positions, N, nClusters, (int)T, &ClusterList); //print out positions at given T intervals between initial and final.
		
		}
	}
	
	else{
		if ((int)T % 50 == 0){ //if, when dividing the new T by 100, the remainder is 0 (i.e. T is a multiple of 100), print the particle positions to file (snapshots)
			printf("T= %05d\n",(int)T); //04 meaning the number is 4 digits long and is padded with zeros to ensure this.
			int Mean = MeanClustSizes(&ClusterList, nClusters); //call fn to show mean cluster sizes at these intervals and set equal to 'mean'
			int Largest = LargestClustSize(&ClusterList, nClusters);
			int Percol = PercolClustSize(&ClusterList, nClusters, &Positions); //the number of particles in the cluster whose index is assigned for the percolating cluster.
						
			SizesToFile(T, Mean, Largest, Percol);
			
			/*Comment in for interval position print out*/
			//MultiSavePositionsFileName(Positions, N, nClusters, (int)T, &ClusterList); //print out positions at given T intervals between initial and final.
		
		}
	
	}

}


void VolFrac(int NumParticles, int SystemSize){ //Fn to print volume fraction of current system to terminal. Call before proceeding with code to check if is reasonable.

	float VolFrac = 100*(float)NumParticles/((float)SystemSize*(float)SystemSize*(float)SystemSize);
	
	printf("Volume fraction = %.2f %%\n ", (float)VolFrac);
	

}


void MonReactProb(float PMonReact){ //Fn to print the reaction probability in percentage form to terminal. 

	float PercPMonReact = 100*(float)PMonReact; //turn reaction probability variable parameter into percentage form
	
	printf("Monomer reaction probability = %.1f %%\n ", (float)PercPMonReact);
	

}


void PrintParams(int NumParticles, int nClusters, int SystemSize, int FinalTime, float PMonReact){ //fn to print variable parameters to terminal, called at start of sim.

  printf(" Number of monomers = %i \n ", NumParticles);
  printf(" Number of clusters = %i \n ", nClusters);
  printf(" Seed size = %i \n", SEED_LENGTH);
  printf("System size = %i \n ", SystemSize);
  VolFrac(NumParticles, SystemSize);
  printf("Final simulation time, T = %i \n ", FinalTime);
  MonReactProb(PMonReact);
  

}

/********************************************************main*********************************************************************************/

int main(){

	/* Sanity check: To keep things simple, assume that the total number of particles is a multiple of the seed size. */
	if( NUMPARTICLES % SEED_LENGTH )			// Returns '1' (meaning 'true' here) if NUMPARTICLES is not divisible by SEED_LENGTH.
	{
		printf( "Total number of particles must be a multiple of the initial seed size 'SEED_LENGTH'.\n" );
		exit( -1 );
	}

 	int NumClusters = NUMPARTICLES / SEED_LENGTH;
 
  int N = SYSTEMSIZE; //Size of 3D space, using variable parameter from "VariableParameters.h" file.
  int **Positions; //Array of the coordinates of the particles in the system. Since an array is already a pointer (*), we use two asterix to point to the pointer
  Cluster_T *ClusterList; //1d array of all the clusters in system. ClusterList is what we now call on to access the defined structure, Cluster_T.
  double T = 0; //Overall simulation time, intialised to 0. Double means double precision for floating point number.
  int PreviousPercolCluster = -1; //Variable for the cluster we are checking is percolated in the last time step - -1 for not-percolating.
  
  
  
  SystemSizeCheck(SYSTEMSIZE, NUMPARTICLES); //before commencing with programme, check that the number of particles chosen can fit in the chosen system size.
  
  MallocParticles(&Positions, NUMPARTICLES); // allocate memory for the indices and coordinates for all individual particles in the system (particle positions). & to get the address of the variable in memory (pass-by-ref to use/change the original value rather than a copy).

  InitialisePercolation(SYSTEMSIZE); //called from Percolation module.

  srand(time(0));  //calls function, srand, from library to generate seed for random number generator. The seed for this (the argument) is another library function to give the current time.
	
  InitialisePositions(Positions, NUMPARTICLES, NumClusters, N); //initialise positions for particles in system. Use the address of 'Positions'in memory, 
	
  /*Terminal print test (CAN COMMENT OUT)*/
  //PrintParticlePositions(&Positions, NUMPARTICLES); 
  
  PrintParams(NUMPARTICLES, NumClusters, SYSTEMSIZE, FINALTIME, P_MON_REACT); //print the variable parameters of the system to terminal before proceeding
  
  MallocClusters(&ClusterList, NumClusters); //allocate the memory for (max) number of clusters in system and their indices.
  
  //MallocClustSizes(Sizes, NUMPARTICLES);
	 
  InitialiseClusterList(&ClusterList, NumClusters); //initialise the members of the ClusterList structure.
  
  const char Filename[] = "InitialParticlePositions.csv";
  SavePositionsFileName(Positions, N, NUMPARTICLES, Filename, &ClusterList);//save initial positions of particles
  
	

  
  //SizesToFile(-1,0,0); //Write header for ClusterSizes.csv file
  WriteSizesToFile(0, ClusterList, NumClusters, Positions, N); //prints header and T=0 data
  WriteBoxCounting(N, NumClusters, &Positions, ClusterList, -1); //Do box counting at very start. Note, our fractal fitting often fails this early on.

  do{	//everything in do loop is for diffusion and aggregation step
	int PreviousPercoClusterSize = PercolClustSize(&ClusterList, NumClusters, &Positions);
	for(int i = 0; i < NumClusters; i++){ //on average attempting to move all clusters in the system in one timestep. However, some may be moved more than once, others may not move at all.
		int ClusterSelect = RandNum()*NumClusters; //Randomly select a cluster from the list of clusters to try to diffuse.
		int SelectedIndex = ClusterSelect; //This will be the selected index from the clusterList, given by the randomly selected cluster from ClusterSelect. This is the 'SelectedLabel' argument in the TrialMove fn.
	
		float p_i = MassProb(&ClusterList, SelectedIndex, FRACT_DIM); //The probability of moving the selected cluster dependent on its mass
		//p_i = 0; //Can comment out - just switches off diffusion.
	
		//printf ("Selected index %d, M_i %i, P_i %f\n", SelectedIndex, M_i, P_i);
		//printf ("Selected index %d, p_i is %f\n", SelectedIndex, p_i);
		
		if (p_i>RandNum()){
			int DirectionMove = 6*RandNum(); //6 directions to move in in 3d, so each DirectionMove will randomly give a value between 0 and 5 inc. int converts the decimal to an integer, rounding down.
		
			if (DirectionMove ==0)
				TrialMove(&ClusterList, NumClusters, SelectedIndex, &Positions, 1, 0, 0); //Try to move the cluster in +ve direction in x plane. It will move if it is not about to overlap onto another cluster
			if (DirectionMove ==1)	
				TrialMove(&ClusterList, NumClusters, SelectedIndex, &Positions, -1, 0, 0);
			if (DirectionMove ==2)	
				TrialMove(&ClusterList, NumClusters, SelectedIndex, &Positions, 0, 1, 0);
			if (DirectionMove ==3)	
				TrialMove(&ClusterList, NumClusters, SelectedIndex, &Positions, 0, -1, 0);	
			if (DirectionMove ==4)	
				TrialMove(&ClusterList, NumClusters, SelectedIndex, &Positions, 0, 0, 1);
			if (DirectionMove ==5)	
				TrialMove(&ClusterList, NumClusters, SelectedIndex, &Positions, 0, 0, -1);
			
				
		}
	}
	// Now doing aggregation (separate to diffusion)
	
	for (int Cluster1 = 0; Cluster1 < NumClusters ; Cluster1++) //loop over all clusters  
		for(int Cluster2 = Cluster1 +1; Cluster2 < NumClusters ; Cluster2++){ //loop over all other clusters, avoiding double counting using 'Cluster1 +1'
			int NumAdjSites = ClustersAdj(&ClusterList, Cluster1, Cluster2, &Positions); //number of adjacent particles between clusters is returned
			if (NumAdjSites >0){
				//printf("\nAdj sites = %i between cluster %i and %i\n", NumAdjSites, Cluster1, Cluster2);//also check the clusters that are adjacent.
				float BondingProb = 1 - pow((1-P_MON_REACT), NumAdjSites); //p is the probability two monomers react with 1 adj site. From 'Prob for cluster agg' file
				
				if (BondingProb>RandNum()){
					AggregateClusters(&ClusterList, Cluster1, Cluster2);
				//	printf("Clusters aggregating: %i, %i\n", Cluster1, Cluster2);
				
				}
			}
	
	}
	
	
	

		
	T += 1.0; //Total time, accumulated by timestep = 1. On average, all clusters have been given a chance to move once in this time. Some will have not moved at all, others more than once.
	
	//WriteSizesToFile(T, ClusterList, NumClusters, Positions, N); //write the means and largest cluster sizes to file (at given time intervals in fn) //COMMENT BACK IN AS BEFORE IF NEED BE
	
	int PercolCluster = IndexOfPercolatingCluster(NumClusters, ClusterList, Positions); //returns index of percolating cluster, or -1 if there are none.
	if (PreviousPercolCluster == -1 && PercolCluster != -1){
      
      printf("Percolating cluster index %i at time %05d\n", PercolCluster,(int)T); //can comment out. If statement for identifying the exact point a cluster percolates (goes from -1 to not -1).
       
      //const char filename[] = "GelPoint.csv";
      //SavePositionsFileName(Positions, N, NUMPARTICLES, filename, &ClusterList); //save updated positions of particles
      GelPointPositionsFileName(Positions, N, NUMPARTICLES, (int)T, &ClusterList);
	  WriteBoxCounting(N, NumClusters, &Positions, ClusterList, PercolCluster); //Do box counting at the point of percolation on the percolating cluster.
	  WriteBoxCounting(N, NumClusters, &Positions, ClusterList, -1);
	  const char filename[] = "ClusterMassDistributions.csv";
	  AllClusterSizes(filename, &ClusterList, NumClusters); //get cluster size distribution at the percolation point. Output to .csv.
	  PrintClusterInfo(&ClusterList, NumClusters); //printed at end of simulation, so will show only the final cluster indices and the contained particle indices
  	
	} 
  
  PreviousPercolCluster = PercolCluster;
  //WritePercolSizesToFile(T, ClusterList, Positions, N, PercolCluster);
  WriteSizesToFile(T, ClusterList, NumClusters, Positions, N); //write the means and largest cluster sizes and percolating cluster size to file (at given time intervals in fn)

  int PercoClusterSize = PercolClustSize(&ClusterList, NumClusters, &Positions);
  if(PercoClusterSize == NumClusters && PreviousPercoClusterSize < NumClusters){ //if the percolation cluster size = system number of particles, do 'final' box counting. Means we do box counting on the largest cluster size, even if we do not reach tf. Only prints once because of < condition.
	  WriteBoxCounting(N, NumClusters, &Positions, ClusterList, -1);
	  
	  const char filename[] = "UpdatedParticlePositions.csv";
	  SavePositionsFileName(Positions, N, NUMPARTICLES, filename, &ClusterList); //save updated positions of particles at the moment the maximal cluster size is reached. 
	  printf("System fully crosslinked/maximal cluster size reached at time %05d\n", (int)T);
  }


  }while(T<FINALTIME); //Final time defined in VariableParameters file.

  
  SizesToFile(-1,0,0,0); //add new line at end of clustersizes file to separate appended results on next run.
  
  /*Terminal print test (CAN COMMENT OUT)*/
  PrintClusterInfo(&ClusterList, NumClusters); //printed at end of simulation, so will show only the final cluster indices and the contained particle indices
  
  CheckParticleIndices(&ClusterList, NUMPARTICLES, NumClusters);
  
  CheckOverlapping(Positions, NUMPARTICLES); //Only reading positions, not changing them, so only need to access 2d array (** in fn, no & needed here)
  
  // WriteBoxCounting(N, NumClusters, &Positions, ClusterList, -1); final time box counting, comment in.
  
  //const char filename[] = "UpdatedParticlePositions.csv";
  //SavePositionsFileName(Positions, N, NUMPARTICLES, filename, &ClusterList); //save updated positions of particles at t final.
  
  //WriteBoxCounting(N, Positions);   
  
  FreeMemory(&Positions, &ClusterList, NUMPARTICLES, NumClusters);
  FinalisePercolation(); //free up memory from percolation algorithm.

  return 0; //for end of 'main'
  

}
