//
// Implementation of the 'Percolation' module.
//

// Include the header file for this module.
#include "Percolation.h"


//
// Initialises any memory / quantities used by the percolation module.
//
void InitialisePercolation( int BoxEdge ) //called with SYSTEMSIZE in ClusterVar.c.
{
    // Allocate memory for each of the indicator arrays.
    xPerc = (int*) malloc( sizeof(int)*BoxEdge );    
    yPerc = (int*) malloc( sizeof(int)*BoxEdge );
    zPerc = (int*) malloc( sizeof(int)*BoxEdge ); 

    // Quick check that the memory was allocated; warn if there was a problem, but otherwise do nothing
    // (will probably soon seg fault with this as one of the last messages output).
    if( xPerc==NULL || yPerc==NULL || zPerc==NULL )
        printf( "Could not allocate memory for the percolation indicator arrays.\n" );

    // Also store the system size, for convenience.
    percXYZ = BoxEdge;
 }


//
// Returns the index of the cluster that percolated, or -1 if there was no percoating cluster.
//
int IndexOfPercolatingCluster( int NumClusters, Cluster_T *Clusters, int **Positions )
{
    // Loop through all clusters, skipping those that have fewer particles than the box size (which cannot percolate) - optimisation.
    for( int c=0; c<NumClusters; c++ )
        if( Clusters[c].NumParts >= percXYZ )
        {
            // Clear the indicator arrays.
            for( int i=0; i<percXYZ; i++ ) xPerc[i] = yPerc[i] = zPerc[i] = 0;

            // Loop through all particles in this cluster, setting the indicator arrays as per the (x,y,z) coordinates.
            for( int n=0; n<Clusters[c].NumParts; n++ )
            {
                int p = Clusters[c].Indices[n];         // Index of particle "n" in cluster "c".

                xPerc[ Positions[p][0] ] = 1;           // Set the indicator to "1" for the x-coord of this particle - particles in cluster "ticked off" and added to array in this dimension.
                yPerc[ Positions[p][1] ] = 1;           // Sim. for y and z.
                zPerc[ Positions[p][2] ] = 1;
            }

            // Did this cluster percolate? Here, "percolated" means simultaneously spanning all 3 dimensions.
            int percolated = 1;
            for( int i=0; i<percXYZ; i++ ) //read through the values in the indicator array.
                if( xPerc[i]==0 || yPerc[i]==0 || zPerc[i]==0 )
                {
                    percolated = 0;
                    break; //stop checking the array at the first 0 in any indicator array dimension - from this point, we know the cluster cannot have percolated. Breaks l53 for loop.
                }

            // If percolated==0 at this point, then this cluster is not percolating; try the next one.
            if( percolated==0 ) continue;       // Continues to the next value of 'c'.

            // If here, this cluster percolates - return with its index. Note this assumes only one cluster percolates at a time,
            // which seems reasonable for any real simulation.
            return c; //Expect to be 0 if cluster percolated. when the code merges clusters, it always uses the lower index for the new, bigger cluster, because of the way we loop through the clusters. So when there is only one cluster left, it must have an index of 0, i.e. the first cluster (cluster indexing starts from 0).
        }

    // If still here, no clusters percolated, so return -1.
    return -1;
}





         



//
// Frees up any resources used by the percolation module.
//
void FinalisePercolation()
{
    free( xPerc );
    free( yPerc );
    free( zPerc );
}

